﻿' arduinoVBserial
' Arduino and Visual Basic: Receiving Data From the Arduino
' A simple example of sending and receiving data to and from an Arduino
' See martyncurrey.com
'

Imports System
Imports System.IO.Ports



Public Class Form1
    ' Global variables
    Dim comPORT As String
    Dim receivedData As String = ""
    Dim connected As Boolean = False
    Dim count = 0

    ' When the program starts; make sure the timer if off (not really needed) and add the available COM ports to the COMport drop down list
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Timer1.Enabled = False
        populateCOMport()
    End Sub

    'The refresh button updates the COMport list
    Private Sub refreshCOM_BTN_Click(sender As Object, e As EventArgs) Handles refreshCOM_CB_BTN.Click
        SerialPort1.Close()
        populateCOMport()
    End Sub

    Private Sub populateCOMport()
        comPORT = ""
        comPort_ComboBox.Items.Clear()
        For Each sp As String In My.Computer.Ports.SerialPortNames
            comPort_ComboBox.Items.Add(sp)
        Next
    End Sub

    Private Sub comPort_ComboBox_SelectedIndexChanged(sender As Object, e As EventArgs) Handles comPort_ComboBox.SelectedIndexChanged
        If (comPort_ComboBox.SelectedItem <> "") Then
            comPORT = comPort_ComboBox.SelectedItem
        End If
    End Sub

    ' When the Connect button is clicked; if a COM port has been selected, connect and send out a HELLO message.
    ' Then wait for the Arduino to respond with its own HELLO.
    ' When the HELLO is received we are connected; change the button text to Dis-connect.
    Private Sub connect_BTN_Click(sender As Object, e As EventArgs) Handles connect_BTN.Click
        comPORT = comPort_ComboBox.SelectedItem
        If (connect_BTN.Text = "Connect") Then
            If (comPORT <> "") Then
                SerialPort1.Close()
                SerialPort1.PortName = comPORT
                SerialPort1.BaudRate = 9600
                SerialPort1.DataBits = 8
                SerialPort1.Parity = Parity.None
                SerialPort1.StopBits = StopBits.One
                SerialPort1.Handshake = Handshake.None
                SerialPort1.Encoding = System.Text.Encoding.Default
                SerialPort1.ReadTimeout = 10000

                SerialPort1.Open()

                'See if the Arduino is there
                count = 0
                SerialPort1.WriteLine("<HELLO>")
                connect_BTN.Text = "Connecting..."
                connecting_Timer.Enabled = True
            Else
                MsgBox("Select a COM port first")
            End If
        Else
            'connect_BTN.Text = "Dis-connect"
            'close the connection a reset the button and timer label
            Timer1.Enabled = False
            Timer_LBL.Text = "Timer: OFF"
            SerialPort1.Close()
            connected = False
            connect_BTN.Text = "Connect"
            populateCOMport()
        End If


    End Sub

    'The connecting_Timer waits for the Arduino to say HELLO.
    ' If HELLO is not received in 2 seconds display an error message.
    ' The connecting_Timer is only used for connecting
    Private Sub connecting_Timer_Tick(sender As Object, e As EventArgs) Handles connecting_Timer.Tick
        connecting_Timer.Enabled = False
        count = count + 1

        If (count <= 8) Then
            receivedData = receivedData & ReceiveSerialData()

            If (Microsoft.VisualBasic.Left(receivedData, 5) = "HELLO") Then
                'if we get an HELLO from the Arduino then we are connected
                connected = True
                connect_BTN.Text = "Dis-connect"
                Timer1.Enabled = True
                Timer_LBL.Text = "Timer: ON"
                receivedData = ReceiveSerialData()
                receivedData = ""
                SerialPort1.WriteLine("<START>")

            Else
                'start the timer again and keep waiting for a signal from the Arduino
                connecting_Timer.Enabled = True
            End If


        Else
            'time out (8 * 250 = 2 seconds)
            RichTextBox1.Text &= vbCrLf & "ERROR" & vbCrLf & "Can not connect" & vbCrLf
            connect_BTN.Text = "Connect"
            populateCOMport()
        End If



    End Sub





    'After a connection is made the main timer waits for data from the Arduino
    'Only OK1, OK2 or OK3 is available.
    Private Sub Timer1_Tick(sender As Object, e As EventArgs) Handles Timer1.Tick
        receivedData = ReceiveSerialData()
        RichTextBox1.Text &= receivedData

        Dim tmp As String
        tmp = Microsoft.VisualBasic.Left(receivedData, 3)

        If (tmp = "OK1") Then
            cmd1_BTN.Enabled = True
        End If

        If (tmp = "OK2") Then
            cmd2_BTN.Enabled = True
        End If

        If (tmp = "OK3") Then
            cmd3_BTN.Enabled = True
        End If

    End Sub

    Function ReceiveSerialData() As String
        Dim Incoming As String
        Try
            Incoming = SerialPort1.ReadExisting()
            If Incoming Is Nothing Then
                Return "nothing" & vbCrLf
            Else
                Return Incoming
            End If
        Catch ex As TimeoutException
            Return "Error: Serial Port read timed out."
        End Try

    End Function


    'Clear the RecievedDtaa test box
    Private Sub clear_BTN_Click(sender As Object, e As EventArgs) Handles clear_BTN.Click
        RichTextBox1.Text = ""
    End Sub




    'LED buttons
    ' This is a basic example and to keep the code simple each button has a separate function. The code could be reduced by using the sender object 
    'If the Arduino is not connected a nessage box display an error message
    Private Sub Pin4_BTN_Click(sender As Object, e As EventArgs) Handles Pin4_BTN.Click
        If (connected) Then
            If (Pin4_BTN.Text = "4 OFF") Then
                SerialPort1.WriteLine("<P004ON>")
                Pin4_BTN.Text = "4 ON"
            Else
                SerialPort1.WriteLine("<P004OF>")
                Pin4_BTN.Text = "4 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub

    Private Sub Pin5_BTN_Click(sender As Object, e As EventArgs) Handles Pin5_BTN.Click
        If (connected) Then
            If (Pin5_BTN.Text = "5 OFF") Then
                SerialPort1.WriteLine("<P005ON>")
                Pin5_BTN.Text = "5 ON"
            Else
                SerialPort1.WriteLine("<P005OF>")
                Pin5_BTN.Text = "5 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub

    Private Sub Pin6_BTN_Click(sender As Object, e As EventArgs) Handles Pin6_BTN.Click
        If (connected) Then
            If (Pin6_BTN.Text = "6 OFF") Then
                SerialPort1.WriteLine("<P006ON>")
                Pin6_BTN.Text = "6 ON"
            Else
                SerialPort1.WriteLine("<P006OF>")
                Pin6_BTN.Text = "6 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub

    Private Sub Pin7_BTN_Click(sender As Object, e As EventArgs) Handles Pin7_BTN.Click
        If (connected) Then
            If (Pin7_BTN.Text = "7 OFF") Then
                SerialPort1.WriteLine("<P007ON>")
                Pin7_BTN.Text = "7 ON"
            Else
                SerialPort1.WriteLine("<P007OF>")
                Pin7_BTN.Text = "7 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub

    Private Sub Pin8_BTN_Click(sender As Object, e As EventArgs) Handles Pin8_BTN.Click
        If (connected) Then
            If (Pin8_BTN.Text = "8 OFF") Then
                SerialPort1.WriteLine("<P008ON>")
                Pin8_BTN.Text = "8 ON"
            Else
                SerialPort1.WriteLine("<P008OF>")
                Pin8_BTN.Text = "8 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub

    Private Sub Pin12_BTN_Click(sender As Object, e As EventArgs) Handles Pin12_BTN.Click
        If (connected) Then
            If (Pin12_BTN.Text = "12 OFF") Then
                SerialPort1.WriteLine("<P012ON>")
                Pin12_BTN.Text = "12 ON"
            Else
                SerialPort1.WriteLine("<P012OF>")
                Pin12_BTN.Text = "12 OFF"
            End If
        Else
            MsgBox("Not connected")
        End If
    End Sub




    'Command buttons
    Private Sub cmd1_BTN_Click(sender As Object, e As EventArgs) Handles cmd1_BTN.Click
        If (connected) Then
            SerialPort1.WriteLine("<CMD01>")
            cmd1_BTN.Enabled = False
        End If
    End Sub

    Private Sub cmd2_BTN_Click(sender As Object, e As EventArgs) Handles cmd2_BTN.Click
        If (connected) Then
            SerialPort1.WriteLine("<CMD02>")
            cmd2_BTN.Enabled = False
        End If
    End Sub

    Private Sub cmd3_BTN_Click(sender As Object, e As EventArgs) Handles cmd3_BTN.Click
        If (connected) Then
            SerialPort1.WriteLine("<CMD03>")
            cmd3_BTN.Enabled = False
        End If
    End Sub




    'RGB sliders
    Private Sub red_TrackBar_Scroll(sender As Object, e As EventArgs) Handles red_TrackBar.Scroll
        updateRGBlbl()
    End Sub

    Private Sub green_TrackBar_Scroll(sender As Object, e As EventArgs) Handles green_TrackBar.Scroll
        updateRGBlbl()
    End Sub

    Private Sub blue_TrackBar_Scroll(sender As Object, e As EventArgs) Handles blue_TrackBar.Scroll
        updateRGBlbl()
    End Sub

    Private Sub updateRGBlbl()
        slider_LBL.Text = "R:" & Format(red_TrackBar.Value, "000") & "  G:" & Format(green_TrackBar.Value, "000") & "  B:" & Format(blue_TrackBar.Value, "000")
        If (connected) Then
            Dim data As String
            data = "<T" & Format(red_TrackBar.Value, "000") & Format(green_TrackBar.Value, "000") & Format(blue_TrackBar.Value, "000") & ">"
            SerialPort1.WriteLine(data)
        End If

    End Sub






    'Navigation buttons
    Private Sub up_BTN_MouseDown(sender As Object, e As MouseEventArgs) Handles up_BTN.MouseDown
        If (connected) Then
            SerialPort1.WriteLine("<NUON>")
        End If
    End Sub

    Private Sub up_BTN_MouseUp(sender As Object, e As MouseEventArgs) Handles up_BTN.MouseUp
        If (connected) Then
            SerialPort1.WriteLine("<NUOF>")
        End If
    End Sub

    Private Sub down_BTN_MouseDown(sender As Object, e As MouseEventArgs) Handles down_BTN.MouseDown
        If (connected) Then
            SerialPort1.WriteLine("<NDON>")
        End If
    End Sub

    Private Sub down_BTN_MouseUp(sender As Object, e As MouseEventArgs) Handles down_BTN.MouseUp
        If (connected) Then
            SerialPort1.WriteLine("<NDOF>")
        End If
    End Sub

    Private Sub left_BTN_MouseDown(sender As Object, e As MouseEventArgs) Handles left_BTN.MouseDown
        If (connected) Then
            SerialPort1.WriteLine("<NLON>")
        End If
    End Sub

    Private Sub left_BTN_MouseUp(sender As Object, e As MouseEventArgs) Handles left_BTN.MouseUp
        If (connected) Then
            SerialPort1.WriteLine("<NLOF>")
        End If
    End Sub

    Private Sub right_BTN_MouseDown(sender As Object, e As MouseEventArgs) Handles right_BTN.MouseDown
        If (connected) Then
            SerialPort1.WriteLine("<NRON>")
        End If
    End Sub

    Private Sub right_BTN_MouseUp(sender As Object, e As MouseEventArgs) Handles right_BTN.MouseUp
        If (connected) Then
            SerialPort1.WriteLine("<NROF>")
        End If
    End Sub


    'The OK button acts as soon as the button is clicked. It does not wait for the user to release the button
    'There is no of for the OK button
    Private Sub ok_BTN_MouseDown(sender As Object, e As MouseEventArgs) Handles ok_BTN.MouseDown
        If (connected) Then
            SerialPort1.WriteLine("<NKOK>")
        End If

    End Sub



    'Add the start and end markers to the contents of the text box and then send to the Arduino.
    'Does not clear the contents
    Private Sub send_BTN_Click(sender As Object, e As EventArgs) Handles send_BTN.Click
        If (connected) Then
            Dim tmp As String
            tmp = "<" & send_TB.Text & ">"
            SerialPort1.WriteLine(tmp)
        End If
    End Sub






End Class
